from Products.CMFCore.utils import _checkPermission

from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
from Products.SecureMailHost.SecureMailHost import EMAIL_RE

from Products.listen.interfaces import IMemberSubscriptionList
from Products.listen.interfaces import IMailingList
from Products.listen.lib.browser_utils import encode
from Products.listen.permissions import SubscribeSelf

from Products.listen.i18n import _

class MailingListView(BrowserView):
    """A basic view of a mailing list"""

    # We certainly need some sort of form controller for z3
    template = ZopeTwoPageTemplateFile('mailinglist_view.pt')
    
    def __call__(self):
        # XXX: This is what we get when we don't have form controller ;)
        sub_action = self.request.get('subscribe_member', None)
        unsub_action = self.request.get('unsubscribe_member', None)
        email_action = self.request.get('subscribe_email', None)
        stop_recvmail_action = self.request.get('stop_recvmail', None)
        start_recvmail_action = self.request.get('start_recvmail', None)
        self.request.set('enable_border', True)
        self.errors = errors = {}
        if sub_action:
            self.subscribe()
            self.request.set('portal_status_message', 'You have been subscribed')
        elif unsub_action:
            self.unsubscribe()
            self.request.set('portal_status_message', 'You have been unsubscribed')
        elif email_action:
            address = self.request.get('email_address', None)
            if not address:
                errors['email_address'] = _('An email address is required')
            elif EMAIL_RE.match(address) is None:
                errors['email_address'] = _('This email address is invalid')
            elif address in IMemberSubscriptionList(self.context).subscribers:
                errors['email_address'] = \
                                 _('This email address is already subscribed')
            else:
                # everything is OK, send a request mail
                request_mail = \
"""From: %s
To: %s
Subject: %s

"""%(address, self.address(), self.subscribe_keyword())
                self.request.set('Mail', request_mail)
                self.context.requestMail(self.request)
                self.request.set('portal_status_message', 'Subscription request sent')
                # Blank the email field to avoid the postback
                self.request.set('email_address', '')
        elif stop_recvmail_action:
            self.stop_recvmail()
        elif start_recvmail_action:
            self.start_recvmail()
        return self.template()

    def Title(self):
        return encode(self.context.title, self.context)

    def Description(self):
        return encode(self.context.description, self.context)

    def address(self):
        return encode(self.context.mailto, self.context)

    # XXX: Perhaps these strings should be done in the template with tal:conditions
    def moderated(self):
        if self.context.moderated:
            return _(u'description-moderated', 'This list is moderated.')
        else:
            return _(u'description-unmoderated', 'This list is not moderated.')

    def archived(self):
        archived = self.context.archived
        if archived == 'plain text':
            return _(u'description-text-archived', 'Messages to this list are archived.')
        elif archived == 'with attachments':
            return _(u'description-all-archived', 'Messages to this list and all attachments are archived.')
        return _(u'description-not-archived', 'Messages to this list are not archived.')

    def closed(self):
        if self.context.closed:
            return _(u'description-closed', 'Only subscribers may post to this list.')
        else:
            return _(u'description-unclosed', 'Anyone may post to this list.')

    def subscribe_keyword(self):
        # Mailboxer stores the subject line keyword used for subscribing as
        # a property
        return self.context.getValueFor('subscribe')

    def unsubscribe_keyword(self):
        # Mailboxer stores the subject line keyword used for unsubscribing as
        # a property
        return self.context.getValueFor('unsubscribe')

    def subscribe(self):
        sub = IMemberSubscriptionList(self.context)
        sub.subscribeMember()

    def unsubscribe(self):
        sub = IMemberSubscriptionList(self.context)
        sub.unsubscribeMember()

    def isSubscribed(self):
        sub = IMemberSubscriptionList(self.context)
        return sub.isMemberSubscribed()

    def stop_recvmail(self):
        sub = IMemberSubscriptionList(self.context)
        sub.stopRecvmailMember()

    def start_recvmail(self):
        sub = IMemberSubscriptionList(self.context)
        sub.startRecvmailMember()

    def isMemberRecvMail(self):
        sub = IMemberSubscriptionList(self.context)
        return sub.isMemberRecvMail()

    def canSubscribe(self):
        return _checkPermission(SubscribeSelf, self.context)

class MailingListArchiveView(BrowserView):
    """A basic view for the mailing list archive.
    """
    def name(self):
        return encode(self.context.title, self.context)
    
    def archive(self):
        return self.context.archive


from zope.formlib import form
from base import EditForm

from zope.app.form.browser import TextAreaWidget
from zope.app.form.browser import ASCIIWidget

class DescriptionWidget(TextAreaWidget):
    width = 40
    height = 5

class MailingListAddForm(form.AddForm):
    """A form for adding MailingList objects.
    """
    form_fields = form.FormFields(IMailingList)
    form_fields['description'].custom_widget = DescriptionWidget
    form_fields['mailto'].custom_widget = ASCIIWidget

class MailingListEditForm(EditForm):
    """A form for editing MailingList objects.
    """
    form_fields = form.FormFields(IMailingList)
    form_fields['description'].custom_widget = DescriptionWidget
    form_fields['mailto'].custom_widget = ASCIIWidget
