from Products.CMFCore.utils import _checkPermission

from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
from Products.SecureMailHost.SecureMailHost import EMAIL_RE

from Products.listen.interfaces import IMemberSubscriptionList
from Products.listen.interfaces import IMailingList
from Products.listen.interfaces import ISubscriptionList
from Products.listen.lib.browser_utils import encode
from Products.listen.permissions import SubscribeSelf
from Products.listen.extras.subscriptions import IInvitationSubscriptionList

from Products.listen.i18n import _

from Products.listen.content.subscriptions import email_regex

def isemail(item):
    return email_regex.match(item)

class ManageMembersView(BrowserView):
    """A basic view of displaying subscribed members and allowed senders """

    # We certainly need some sort of form controller for z3
    template = ZopeTwoPageTemplateFile('manage_membership.pt')
    
    def __call__(self):
        if not self.request.get('save', None): return self.template()
        d = self.request.form
        to_rename = []
        to_remove = []
        to_subscribe = []
        for name, value in d.items():
            if name == 'save' and value == 'save changes': continue
            valuetype, name = name.split('_', 1)
            if valuetype == 'email':
                if name != value:
                    to_rename.append((name, value))
            elif valuetype == 'remove':
                to_remove.append(name)
            elif valuetype == 'subscribed':
                to_subscribe.append(name)
        
        # if we have to rename/subscribe an item that is to be removed
        # we should just remove it
        to_subscribe = list(set(to_subscribe).difference(to_remove))
        to_rename = [(a, b) for a, b in to_rename if a not in to_remove]

        self._remove(to_remove)
        self._subscribe(to_subscribe)
        self._rename(to_rename)

        to_add = d.get('add_email', None)
        if to_add:
            subscribed = d.get('add_subscribed', None)
            self._add(to_add, subscribed)

        return self.template()

    def _add(self, item, subscribed):
        if isemail(item):
            ISubscriptionList(self.context).subscribeAddress(item)
            ISubscriptionList(self.context).stopRecvmail(item)
        else:
            IMemberSubscriptionList(self.context).subscribeMember(item)
            IMemberSubscriptionList(self.context).stopRecvmailMember(item)

        if subscribed:
            if isemail(item):
                IInvitationSubscriptionList(self.context)._addPendingAddress(item)
            else:
                IInvitationSubscriptionList(self.context)._addPendingMember(item)

    def _remove(self, remove_list):
        remove_address = ISubscriptionList(self.context).unsubscribeAddress
        remove_member = IMemberSubscriptionList(self.context).unsubscribeMember
        for item in remove_list:
            if isemail(item):
                remove_address(item)
            else:
                remove_member(item)

    def _subscribe(self, add_list):
        add_address = IInvitationSubscriptionList(self.context)._addPendingAddress
        add_member = IInvitationSubscriptionList(self.context)._addPendingMember
        stop_address = ISubscriptionList(self.context).stopRecvmail
        stop_member = IMemberSubscriptionList(self.context).stopRecvmailMember
        subscribers = self.subscribers()
        for item in add_list:
            if isemail(item):
                if item in ISubscriptionList(self.context).subscribers: continue
                add_address(item)
            else:
                if IMemberSubscriptionList(self.context).getEmailFromId(item)\
                        in ISubscriptionList(self.context).subscribers: continue
                add_member(item)

        # now we should remove any members that are not present in the subscribe
        # lists
        for s in subscribers:
            if s not in add_list:
                if isemail(s):
                    stop_address(s)
                else:
                    stop_member(s)

    def _rename(self, rename_list):
        add_address = ISubscriptionList(self.context).subscribeAddress
        add_member = IMemberSubscriptionList(self.context).subscribeMember
        remove_address = ISubscriptionList(self.context).unsubscribeAddress
        remove_member = IMemberSubscriptionList(self.context).unsubscribeMember
        for old_name, new_name in rename_list:
            if isemail(old_name):
                is_subscribed = ISubscriptionList(self.context).isRecvMail(old_name)
                remove_address(old_name)
                add_address(new_name)
                if not is_subscribed:
                    ISubscriptionList(self.context).stopRecvmail(new_name)
            else:
                is_subscribed = IMemberSubscriptionList(self.context).isRecvMail(old_name)
                remove_member(old_name)
                add_member(new_name)
                if not is_subscribed:
                    IMemberSubscriptionList(self.context).stopRecvmailMember(new_name)

    def Title(self):
        return 'Manage Allowed Senders'

    def Description(self):
        return 'Manage Allowed Senders'

    def subscribers(self):
        addresses = list(ISubscriptionList(self.context).getAllowedSenders())
        members = IMemberSubscriptionList(self.context).subscribed_members
        for member in members:
            email = IMemberSubscriptionList(self.context).getEmailFromId(member)
            if email in addresses:
                addresses.remove(email)
        return tuple(addresses) + members

    def isRecvMail(self, item):
        if isemail(item):
            return ISubscriptionList(self.context).isRecvMail(item)
        else:
            return IMemberSubscriptionList(self.context).isMemberRecvMail(item)

    def _convert_to_email(self, item):
        if isemail(item):
            return item
        else:
            return IMemberSubscriptionList(self.context).getEmailFromId(item)

    def isPending(self, item):
        pending_emails = map(self._convert_to_email,
                             IInvitationSubscriptionList(self.context).pending_members)
        return self._convert_to_email(item) in pending_emails

    def pendingDT(self, item):
        return IInvitationSubscriptionList(self.context).pending_data[item]
